@@ -38,20 +38,28 @@
 #include "message.h"
 #include "intl.h"
 
+/** Define this if you want to use the layout cache to speed up output.
+ *  Appears to be buggy, see bug #307320. */
+#undef LAYOUT_CACHE
+
 static PangoContext* pango_context = NULL;
 
+struct _DiaFont 
+{
+    GObject parent_instance;
+
+    PangoFontDescription* pfd;    
+    /* mutable */ char* legacy_name;    
+};
+
+
 /* This is the global factor that says what zoom factor is 100%.  It's
  * normally 20.0 (and likely to stay that way).  It is defined by how many
  * pixels one cm is represented as.
  */
 static real global_zoom_factor = 20.0;
 
-static int
-compare(const void *p1, const void *p2) {
-  return g_ascii_strcasecmp(pango_font_family_get_name(*(PangoFontFamily **)p1),
-			    pango_font_family_get_name(*(PangoFontFamily **)p2));
-}
-
+#if 0
 /* For debugging: Sort families */
 static int
 cmp_families (const void *a, const void *b)
@@ -75,7 +83,9 @@ list_families()
   for (i = 0; i < nfamilies; i++) {
     puts(pango_font_family_get_name(families[i]));
   }
+  g_free(families);
 }
+#endif
 
 static void
 dia_font_check_for_font(int font) {
@@ -287,7 +297,8 @@ dia_font_new_from_style(DiaFontStyle style, real height)
   dia_pfd_set_slant(pfd,DIA_FONT_STYLE_GET_SLANT(style));
   dia_pfd_set_size(pfd,height);
   
-  retval = DIA_FONT(g_type_create_instance(dia_font_get_type()));
+  retval = DIA_FONT(g_object_new(DIA_TYPE_FONT, NULL));
+  
   retval->pfd = pfd;
   retval->legacy_name = NULL;
   return retval;
@@ -306,7 +317,6 @@ dia_font_finalize(GObject* object)
 {
     DiaFont* font;
     font = DIA_FONT(object);
-    
     if (font->pfd) pango_font_description_free(font->pfd);
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
@@ -354,6 +364,12 @@ dia_font_get_family(const DiaFont* font)
   return pango_font_description_get_family(font->pfd);
 }
 
+G_CONST_RETURN PangoFontDescription *
+dia_font_get_description(const DiaFont* font)
+{
+  return font->pfd;
+}
+
 real
 dia_font_get_height(const DiaFont* font)
 {
@@ -521,6 +537,10 @@ layout_cache_equals(gconstpointer e1, gconstpointer e2)
   LayoutCacheItem *i1 = (LayoutCacheItem*)e1,
     *i2 = (LayoutCacheItem*)e2;
 
+  /* don't try strcmp() with null pointers ... */
+  if (!i1->string || i2->string)
+    return FALSE;
+  
   return strcmp(i1->string, i2->string) == 0 &&
     pango_font_description_equal(i1->font->pfd, i2->font->pfd);
 }
@@ -603,6 +623,7 @@ dia_font_build_layout(const char* string, DiaFont* font, real height)
     guint length;
     gchar *desc = NULL;
 
+#ifdef LAYOUT_CACHE
     LayoutCacheItem *cached, *item;
 
     layout_cache_last_use = time(0);
@@ -620,10 +641,12 @@ dia_font_build_layout(const char* string, DiaFont* font, real height)
        */
       g_timeout_add(10*1000, layout_cache_cleanup, (gpointer)layoutcache);
     }
+#endif
 
     height *= 0.7;
     dia_font_set_height(font, height);
 
+#ifdef LAYOUT_CACHE
     item = g_new0(LayoutCacheItem,1);
     item->string = g_strdup(string);
     item->font = font;
@@ -638,7 +661,9 @@ dia_font_build_layout(const char* string, DiaFont* font, real height)
       return cached->layout;
     }
 
-    dia_font_ref(font);
+    item->font = dia_font_copy(font);
+    dia_font_ref(item->font);
+#endif
 
     /* This could should account for DPI, but it doesn't do right.  Grrr...
     {
@@ -672,10 +697,12 @@ dia_font_build_layout(const char* string, DiaFont* font, real height)
     pango_layout_set_justify(layout,FALSE);
     pango_layout_set_alignment(layout,PANGO_ALIGN_LEFT);
   
+#ifdef LAYOUT_CACHE
     item->layout = layout;
     g_object_ref(layout);
     item->usecount = 1;
     g_hash_table_replace(layoutcache, item, item);
+#endif
 
     return layout;
 }
@@ -836,6 +863,8 @@ dia_font_scaled_build_layout(const char* string, DiaFont* font,
 /**
  * Compatibility with older files out of pre Pango Time. 
  * Make old files look as similar as possible
+ * List should be kept alphabetically sorted by oldname, in case of
+ * duplicates the one with the preferred newname comes first.
  *
  * FIXME: DIA_FONT_FAMILY_ANY in the list below does mean noone knows better
  */
@@ -844,7 +873,6 @@ static struct _legacy_font {
   gchar*       newname;
   DiaFontStyle style;   /* the DIA_FONT_FAMILY() is used as falback only */
 } legacy_fonts[] = {
-  /* these _MUST_ be sorted alphabetical */
   { "AvantGarde-Book",        "AvantGarde", DIA_FONT_SERIF },
   { "AvantGarde-BookOblique", "AvantGarde", DIA_FONT_SERIF | DIA_FONT_OBLIQUE },
   { "AvantGarde-Demi",        "AvantGarde", DIA_FONT_SERIF | DIA_FONT_DEMIBOLD },
@@ -857,9 +885,12 @@ static struct _legacy_font {
   { "BousungEG-Light-GB", "BousungEG-Light-GB", DIA_FONT_FAMILY_ANY },
   { "Courier",             "monospace", DIA_FONT_MONOSPACE },
   { "Courier",             "Courier New", DIA_FONT_MONOSPACE },
+  { "Courier-Bold",        "monospace", DIA_FONT_MONOSPACE | DIA_FONT_BOLD },
   { "Courier-Bold",        "Courier New", DIA_FONT_MONOSPACE | DIA_FONT_BOLD },
-  { "Courier-BoldOblique", "Courier New", DIA_FONT_MONOSPACE | DIA_FONT_OBLIQUE | DIA_FONT_BOLD },
-  { "Courier-Oblique",     "Courier New", DIA_FONT_MONOSPACE | DIA_FONT_OBLIQUE },
+  { "Courier-BoldOblique", "monospace", DIA_FONT_MONOSPACE | DIA_FONT_ITALIC | DIA_FONT_BOLD },
+  { "Courier-BoldOblique", "Courier New", DIA_FONT_MONOSPACE | DIA_FONT_ITALIC | DIA_FONT_BOLD },
+  { "Courier-Oblique",     "monospace", DIA_FONT_MONOSPACE | DIA_FONT_ITALIC},
+  { "Courier-Oblique",     "Courier New", DIA_FONT_MONOSPACE | DIA_FONT_ITALIC },
   { "Dotum", "Dotum", DIA_FONT_FAMILY_ANY },
   { "GBZenKai-Medium", "GBZenKai-Medium", DIA_FONT_FAMILY_ANY }, 
   { "GothicBBB-Medium", "GothicBBB-Medium", DIA_FONT_FAMILY_ANY },
@@ -867,13 +898,16 @@ static struct _legacy_font {
   { "Headline", "Headline", DIA_FONT_FAMILY_ANY },
   { "Helvetica",             "sans", DIA_FONT_SANS },
   { "Helvetica",             "Arial", DIA_FONT_SANS },
+  { "Helvetica-Bold",        "sans", DIA_FONT_SANS | DIA_FONT_BOLD },
   { "Helvetica-Bold",        "Arial", DIA_FONT_SANS | DIA_FONT_BOLD },
-  { "Helvetica-BoldOblique", "Arial", DIA_FONT_SANS | DIA_FONT_BOLD | DIA_FONT_OBLIQUE },
+  { "Helvetica-BoldOblique", "sans", DIA_FONT_SANS | DIA_FONT_BOLD | DIA_FONT_ITALIC },
+  { "Helvetica-BoldOblique", "Arial", DIA_FONT_SANS | DIA_FONT_BOLD | DIA_FONT_ITALIC },
   { "Helvetica-Narrow",             "Arial Narrow", DIA_FONT_SANS | DIA_FONT_MEDIUM },
   { "Helvetica-Narrow-Bold",        "Arial Narrow", DIA_FONT_SANS | DIA_FONT_DEMIBOLD },
   { "Helvetica-Narrow-BoldOblique", "Arial Narrow", DIA_FONT_SANS | DIA_FONT_MEDIUM | DIA_FONT_OBLIQUE },
   { "Helvetica-Narrow-Oblique",     "Arial Narrow", DIA_FONT_SANS | DIA_FONT_MEDIUM | DIA_FONT_OBLIQUE },
-  { "Helvetica-Oblique",     "Arial", DIA_FONT_SANS | DIA_FONT_OBLIQUE },
+  { "Helvetica-Oblique",     "sans", DIA_FONT_SANS | DIA_FONT_ITALIC },
+  { "Helvetica-Oblique",     "Arial", DIA_FONT_SANS | DIA_FONT_ITALIC },
   { "MOESung-Medium", "MOESung-Medium", DIA_FONT_FAMILY_ANY },
   { "NewCenturySchoolbook-Bold",       "Century Schoolbook SWA", DIA_FONT_SERIF | DIA_FONT_BOLD },
   { "NewCenturySchoolbook-BoldItalic", "Century Schoolbook SWA", DIA_FONT_SERIF | DIA_FONT_BOLD | DIA_FONT_ITALIC },
@@ -883,12 +917,15 @@ static struct _legacy_font {
   { "Palatino-BoldItalic", "Palatino", DIA_FONT_FAMILY_ANY | DIA_FONT_BOLD | DIA_FONT_ITALIC }, 
   { "Palatino-Italic",     "Palatino", DIA_FONT_FAMILY_ANY | DIA_FONT_ITALIC }, 
   { "Palatino-Roman",      "Palatino", DIA_FONT_FAMILY_ANY },
-  { "Ryumin-Light", "Ryumin", DIA_FONT_FAMILY_ANY | DIA_FONT_LIGHT },
-  { "ShanHeiSun-Light", "ShanHeiSun", DIA_FONT_FAMILY_ANY | DIA_FONT_LIGHT },
-  { "Song-Medium", "Song-Medium", DIA_FONT_FAMILY_ANY | DIA_FONT_MEDIUM },
-  { "Symbol", "Symbol", DIA_FONT_SANS | DIA_FONT_MEDIUM },
+  { "Ryumin-Light",      "Ryumin", DIA_FONT_FAMILY_ANY | DIA_FONT_LIGHT },
+  { "ShanHeiSun-Light",  "ShanHeiSun", DIA_FONT_FAMILY_ANY | DIA_FONT_LIGHT },
+  { "Song-Medium",       "Song-Medium", DIA_FONT_FAMILY_ANY | DIA_FONT_MEDIUM },
+  { "Symbol",           "Symbol", DIA_FONT_SANS | DIA_FONT_MEDIUM },
+  { "Times-Bold",       "serif", DIA_FONT_SERIF | DIA_FONT_BOLD }, 
   { "Times-Bold",       "Times New Roman", DIA_FONT_SERIF | DIA_FONT_BOLD },
+  { "Times-BoldItalic", "serif", DIA_FONT_SERIF | DIA_FONT_ITALIC | DIA_FONT_BOLD }, 
   { "Times-BoldItalic", "Times New Roman", DIA_FONT_SERIF | DIA_FONT_ITALIC | DIA_FONT_BOLD },
+  { "Times-Italic",     "serif", DIA_FONT_SERIF | DIA_FONT_ITALIC }, 
   { "Times-Italic",     "Times New Roman", DIA_FONT_SERIF | DIA_FONT_ITALIC },
   { "Times-Roman",      "serif", DIA_FONT_SERIF }, 
   { "Times-Roman",      "Times New Roman", DIA_FONT_SERIF }, 
@@ -898,14 +935,6 @@ static struct _legacy_font {
 };
 
 
-static int
-fonts_compare (const void *key, const void *elem)
-{
-  char* a = (char*)key;
-  char* b = ((struct _legacy_font*)elem)->oldname;
-  return strcmp (a, b);
-}
-
 /**
  * Given a legacy name as stored until Dia-0.90 construct
  * a new DiaFont which is as similar as possible
@@ -915,12 +944,16 @@ dia_font_new_from_legacy_name(const char* name)
 {
   /* do NOT translate anything here !!! */
   DiaFont* retval;
-  struct _legacy_font* found;
+  struct _legacy_font* found = NULL;
   real height = 1.0;
+  int i;
 
-  found = bsearch (name, legacy_fonts,
-		   G_N_ELEMENTS(legacy_fonts), sizeof (struct _legacy_font),
-		   fonts_compare);
+  for (i = 0; i < G_N_ELEMENTS(legacy_fonts); i++) {
+    if (!strcmp(name, legacy_fonts[i].oldname)) {
+      found = &legacy_fonts[i];
+      break;
+    }
+  }
   if (found) {
     retval = dia_font_new (found->newname, found->style, height);
     retval->legacy_name = found->oldname;
@@ -951,10 +984,10 @@ dia_font_get_legacy_name(const DiaFont *font)
     if (0 == g_strcasecmp (legacy_fonts[i].newname, family)) {
       /* match weight and slant */
       DiaFontStyle st = legacy_fonts[i].style;
-      if (   (DIA_FONT_STYLE_GET_SLANT(style) || DIA_FONT_STYLE_GET_WEIGHT(style))
-          == (DIA_FONT_STYLE_GET_SLANT(st) || DIA_FONT_STYLE_GET_WEIGHT(st))) {
+      if ((DIA_FONT_STYLE_GET_SLANT(style) | DIA_FONT_STYLE_GET_WEIGHT(style))
+          == (DIA_FONT_STYLE_GET_SLANT(st) | DIA_FONT_STYLE_GET_WEIGHT(st))) {
         return legacy_fonts[i].oldname; /* exact match */
-      } else if (0 == (DIA_FONT_STYLE_GET_SLANT(st) || DIA_FONT_STYLE_GET_WEIGHT(st))) {
+      } else if (0 == (DIA_FONT_STYLE_GET_SLANT(st) | DIA_FONT_STYLE_GET_WEIGHT(st))) {
         matched_name = legacy_fonts[i].oldname;
         /* 'unmodified' font, continue matching */
       }
