@@ -18,11 +18,20 @@
 #ifndef MESSAGE_H
 #define MESSAGE_H
 
-void message(char *title, const char *format, ...);
+#include <stdarg.h>
+
+typedef void (*MessageInternal)(const char *title, const char *fmt,
+                                va_list *args,  va_list *args2);
+
+void set_message_func(MessageInternal func);
+void message(const char *title, const char *format, ...);
 void message_notice(const char *format, ...);
 void message_warning(const char *format, ...);
 void message_error(const char *format, ...);
 
+/* also declared in dia_dirs.h, where I think it does not belong! --hb */
+const gchar *dia_message_filename (const gchar *filename);
+
 #endif /* MESSAGES_H */
 
 
