@@ -30,29 +30,31 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#ifdef HAVE_DIRENT_H
-#include <dirent.h>
-#endif
 
-#include <parser.h>
-#include <tree.h>
-#if defined(LIBXML_VERSION) && LIBXML_VERSION >= 20000
-#define root children
-#define childs children
-#endif
+#include <libxml/xmlmemory.h>
+#include <libxml/parser.h>
+#include <libxml/tree.h>
+#include "dia_xml_libxml.h"
+#include "dia_xml.h"
 
 #include "plug-ins.h"
 #include "intl.h"
 #include "message.h"
 #include "dia_dirs.h"
 
+#ifdef G_OS_WIN32
+#include <io.h> /* open, close */
+#endif
+
 #if defined(G_OS_WIN32) || defined(__EMX__)
 #  define PLUG_IN_EXT ".dll"
 #  define PLUG_IN_EXT_LEN 4
+#  define USING_LIBTOOL 0
 #else
 /* this one should work on any platform where libtool is used to compile dia */
 #  define PLUG_IN_EXT ".la"
 #  define PLUG_IN_EXT_LEN 3
+#  define USING_LIBTOOL 1
 #endif
 
 struct _PluginInfo {
@@ -71,6 +73,7 @@ struct _PluginInfo {
   PluginUnloadFunc unload_func;
 };
 
+
 static GList *plugins = NULL;
 
 static void     ensure_pluginrc(void);
@@ -103,7 +106,7 @@ dia_plugin_check_version(gint version)
 }
 
 
-/* functiosn for use by dia ... */
+/* functions for use by dia ... */
 
 const gchar *
 dia_plugin_get_filename(PluginInfo *info)
@@ -114,7 +117,7 @@ dia_plugin_get_filename(PluginInfo *info)
 const gchar *
 dia_plugin_get_name(PluginInfo *info)
 {
-  return info->name;
+  return info->name ? info->name : _("???");
 }
 
 const gchar *
@@ -123,6 +126,20 @@ dia_plugin_get_description(PluginInfo *info)
   return info->description;
 }
 
+const gpointer *
+dia_plugin_get_symbol(PluginInfo *info, const gchar* name)
+{
+  gpointer symbol;
+
+  if (!info->module)
+    return NULL;
+
+  if (g_module_symbol (info->module, name, &symbol))
+    return symbol;
+
+  return NULL;
+}
+
 gboolean
 dia_plugin_can_unload(PluginInfo *info)
 {
@@ -186,9 +203,8 @@ find_real_filename(const gchar *filename)
     return NULL;
   }
   
-  str = g_dirname(filename);
-  ret = g_strconcat(str, G_DIR_SEPARATOR_S, scanner->value.v_string,
-		    NULL);
+  str = g_path_get_dirname(filename);
+  ret = g_build_filename(str, scanner->value.v_string, NULL);
   g_free(str);
   g_scanner_destroy(scanner);
   close(fd);
@@ -213,8 +229,11 @@ dia_plugin_load(PluginInfo *info)
   }
   info->module = g_module_open(info->real_filename, G_MODULE_BIND_LAZY);
   if (!info->module) {
-    message_error(_("Could not load plugin `%s'\n%s"), info->filename,
-		  g_module_error());
+    gchar *msg_utf8 = g_locale_to_utf8 (g_module_error(), -1, NULL, NULL, NULL);
+    message_error(_("Could not load plugin '%s'\n%s"), 
+                  info->filename, msg_utf8);
+    /* this is eating the conversion */
+    info->description = msg_utf8;
     return;
   }
 
@@ -226,6 +245,7 @@ dia_plugin_load(PluginInfo *info)
 
     message_error(_("Could not find plugin init function in `%s'"),
 		  info->filename);
+    info->description = g_strdup(_("Missing symbol 'dia_plugin_init'"));
     return;
   }
 
@@ -233,6 +253,15 @@ dia_plugin_load(PluginInfo *info)
     /* plugin displayed an error message */
     g_module_close(info->module);
     info->module = NULL;
+    info->description = g_strdup(_("dia_plugin_init() call failed"));
+    return;
+  }
+
+  /* Corrupt? */
+  if (info->description == NULL) {
+    g_module_close(info->module);
+    info->module = NULL;
+    info->description = g_strdup(_("dia_plugin_init() call failed"));
     return;
   }
 
@@ -279,6 +308,11 @@ dia_register_plugin(const gchar *filename)
       return;
   }
 
+  /* If trying to load libdia, abort */
+  if (strstr(filename, "libdia.")) {
+    return;
+  }
+
   /* set up plugin info structure */
   info = g_new0(PluginInfo, 1);
   info->filename = g_strdup(filename);
@@ -294,46 +328,119 @@ dia_register_plugin(const gchar *filename)
   plugins = g_list_prepend(plugins, info);
 }
 
-void
-dia_register_plugins_in_dir(const gchar *directory)
+static gboolean
+this_is_a_plugin(const gchar *name) 
+{
+#if USING_LIBTOOL
+  gchar *soname,*basename;
+#endif
+  guint len = strlen(name);
+  if (0 != strcmp(&name[len-PLUG_IN_EXT_LEN], PLUG_IN_EXT)) 
+    return FALSE;
+#if USING_LIBTOOL
+  basename = g_strndup(name,len-PLUG_IN_EXT_LEN);
+  soname = g_strconcat(basename,".so",NULL);
+  if (!g_file_test(soname, G_FILE_TEST_IS_REGULAR)) {    
+    g_free(basename);
+    g_free(soname);
+    return FALSE;
+  }
+  g_free(basename);
+  g_free(soname);
+#endif
+  return TRUE;
+}
+
+typedef void (*ForEachInDirDoFunc)(const gchar *name);
+typedef gboolean (*ForEachInDirFilterFunc)(const gchar *name);
+
+static void 
+for_each_in_dir(const gchar *directory, ForEachInDirDoFunc dofunc,
+                ForEachInDirFilterFunc filter)
 {
   struct stat statbuf;
-  struct dirent *dirp;
-  DIR *dp;
+  const char *dentry;
+  GDir *dp;
+  GError *error = NULL;
 
   if (stat(directory, &statbuf) < 0)
     return;
-  if (!S_ISDIR(statbuf.st_mode)) {
-    message_warning(_("`%s' is not a directory"), directory);
-    return;
-  }
 
-  dp = opendir(directory);
+  dp = g_dir_open(directory, 0, &error);
   if (dp == NULL) {
-    message_warning(_("Could not open `%s'"), directory);
+    message_warning(_("Could not open `%s'\n`%s'"), directory, error->message);
+    g_error_free (error);
     return;
   }
 
-  while ((dirp = readdir(dp)) != NULL) {
-    gint len = strlen(dirp->d_name);
+  while ((dentry = g_dir_read_name(dp)) != NULL) {
+    gchar *name = g_strconcat(directory,G_DIR_SEPARATOR_S,dentry,NULL);
 
-    if (len > PLUG_IN_EXT_LEN &&
-	!strcmp(&dirp->d_name[len-PLUG_IN_EXT_LEN], PLUG_IN_EXT)) {
-      gchar *filename = g_strconcat(directory, G_DIR_SEPARATOR_S,
-				    dirp->d_name, NULL);
-
-      dia_register_plugin(filename);
-      g_free(filename);
-    }
+    if (filter(name)) dofunc(name);
+    g_free(name);
   }
-  closedir(dp);
+  g_dir_close(dp);
+}
+
+static gboolean 
+directory_filter(const gchar *name)
+{
+  guint len = strlen(name);
+
+  if (0 == strcmp(G_DIR_SEPARATOR_S ".",
+                  &name[len-strlen(G_DIR_SEPARATOR_S ".")])) return FALSE;
+  if (0 == strcmp(G_DIR_SEPARATOR_S "..",
+                  &name[len-strlen(G_DIR_SEPARATOR_S "..")])) return FALSE;
+
+  if (!g_file_test (name, G_FILE_TEST_IS_DIR))
+    return FALSE;
+
+  return TRUE;
+}
+
+static gboolean 
+dia_plugin_filter(const gchar *name) 
+{
+  gint len = strlen(name);
+
+  if (!g_file_test (name, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_DIR))
+    return FALSE;
+  
+  if (len <= PLUG_IN_EXT_LEN) return FALSE;
+
+  return this_is_a_plugin(name);
+}
+
+static void
+walk_dirs_for_plugins(const gchar *dirname)
+{
+  for_each_in_dir(dirname,walk_dirs_for_plugins,directory_filter);  
+  for_each_in_dir(dirname,dia_register_plugin,dia_plugin_filter);
+}
+
+#define RECURSE (G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S)
+
+void
+dia_register_plugins_in_dir(const gchar *directory)
+{
+  guint reclen = strlen(RECURSE);
+  guint len = strlen(directory);
+
+  if ((len >= reclen) &&
+      (0 == strcmp(&directory[len-reclen],RECURSE))) {
+    gchar *dirbase = g_strndup(directory,len-reclen);
+    for_each_in_dir(dirbase,walk_dirs_for_plugins,directory_filter);
+    g_free(dirbase);
+  };
+  /* intentional fallback. */
+  for_each_in_dir(directory,dia_register_plugin,dia_plugin_filter);
 }
 
 void
 dia_register_plugins(void)
 {
-  gchar *library_path;
-  gchar *lib_dir;
+  const gchar *library_path;
+  const gchar *lib_dir;
 
   library_path = g_getenv("DIA_LIB_PATH");
 
@@ -341,7 +448,7 @@ dia_register_plugins(void)
 
   if (lib_dir != NULL) {
     dia_register_plugins_in_dir(lib_dir);
-    g_free(lib_dir);
+    g_free((char *)lib_dir);
   }
 
   if (library_path != NULL) {
@@ -356,9 +463,9 @@ dia_register_plugins(void)
     library_path = dia_get_lib_directory("dia");
 
     dia_register_plugins_in_dir(library_path);
-    g_free(library_path);
+    g_free((char *)library_path);
   }
-  /* this isn't needed anymore */
+  /* FIXME: this isn't needed anymore */
   free_pluginrc();
 }
 
@@ -406,13 +513,17 @@ ensure_pluginrc(void)
 
   if (pluginrc)
     return;
-
   filename = dia_config_filename("pluginrc");
-  pluginrc = xmlParseFile(filename);
+  if (g_file_test (filename,  G_FILE_TEST_IS_REGULAR))
+    pluginrc = xmlDiaParseFile(filename);
+  else
+    pluginrc = NULL;
+  
   g_free(filename);
 
   if (!pluginrc) {
     pluginrc = xmlNewDoc("1.0");
+    pluginrc->encoding = xmlStrdup("UTF-8");
     xmlDocSetRootElement(pluginrc,
 			 xmlNewDocNode(pluginrc, NULL, "plugins", NULL));
   }
@@ -434,8 +545,12 @@ plugin_load_inhibited(const gchar *filename)
   xmlNodePtr node;
 
   ensure_pluginrc();
-  for (node = pluginrc->root->childs; node != NULL; node = node->next) {
-    CHAR *node_filename;
+  for (node = pluginrc->xmlRootNode->xmlChildrenNode; 
+       node != NULL; 
+       node = node->next) {
+    xmlChar *node_filename;
+
+    if (xmlIsBlankNode(node)) continue;
 
     if (node->type != XML_ELEMENT_NODE || strcmp(node->name, "plugin") != 0)
       continue;
@@ -443,15 +558,18 @@ plugin_load_inhibited(const gchar *filename)
     if (node_filename && !strcmp(filename, node_filename)) {
       xmlNodePtr node2;
 
-      free(node_filename);
-      for (node2 = node->childs; node2 != NULL; node2 = node2->next) {
+      xmlFree(node_filename);
+      for (node2 = node->xmlChildrenNode; 
+           node2 != NULL; 
+           node2 = node2->next) {
+        if (xmlIsBlankNode(node2)) continue;
 	if (node2->type == XML_ELEMENT_NODE &&
 	    !strcmp(node2->name, "inhibit-load"))
 	  return TRUE;
       }
       return FALSE;
     }
-    if (node_filename) free(node_filename);
+    if (node_filename) xmlFree(node_filename);
   }
   return FALSE;
 }
@@ -471,8 +589,12 @@ info_fill_from_pluginrc(PluginInfo *info)
   info->unload_func = NULL;
 
   ensure_pluginrc();
-  for (node = pluginrc->root->childs; node != NULL; node = node->next) {
-    CHAR *node_filename;
+  for (node = pluginrc->xmlRootNode->xmlChildrenNode; 
+       node != NULL; 
+       node = node->next) {
+    xmlChar *node_filename;
+
+    if (xmlIsBlankNode(node)) continue;
 
     if (node->type != XML_ELEMENT_NODE || strcmp(node->name, "plugin") != 0)
       continue;
@@ -480,10 +602,14 @@ info_fill_from_pluginrc(PluginInfo *info)
     if (node_filename && !strcmp(info->filename, node_filename)) {
       xmlNodePtr node2;
 
-      free(node_filename);
-      for (node2 = node->childs; node2 != NULL; node2 = node2->next) {
+      xmlFree(node_filename);
+      for (node2 = node->xmlChildrenNode; 
+           node2 != NULL; 
+           node2 = node2->next) {
 	char *content;
 
+        if (xmlIsBlankNode(node2)) continue;
+
 	if (node2->type != XML_ELEMENT_NODE)
 	  continue;
 	content = xmlNodeGetContent(node2);
@@ -494,11 +620,11 @@ info_fill_from_pluginrc(PluginInfo *info)
 	  g_free(info->description);
 	  info->description = g_strdup(content);
 	}
-	free(content);
+	xmlFree(content);
       }
       break;
     }
-    if (node_filename) free(node_filename);
+    if (node_filename) xmlFree(node_filename);
   }
 }
 
@@ -519,33 +645,47 @@ dia_pluginrc_write(void)
 
     pluginnode = xmlNewNode(NULL, "plugin");
     datanode = xmlNewChild(pluginnode, NULL, "name", info->name);
-    datanode = xmlNewChild(pluginnode, NULL, "description", info->description);
+    /* FIXME: UNICODE_WORK_IN_PROGRESS why is this reencoding necessary ?*/
+ {
+     xmlChar *enc = xmlEncodeEntitiesReentrant(pluginnode->doc,
+                                               info->description);
+     datanode = xmlNewChild(pluginnode, NULL, "description", enc);
+     xmlFree(enc);
+ }
     if (info->inhibit_load)
       datanode = xmlNewChild(pluginnode, NULL, "inhibit-load", NULL);
 
-    for (node = pluginrc->root->childs; node != NULL; node = node->next) {
-      CHAR *node_filename;
+    for (node = pluginrc->xmlRootNode->xmlChildrenNode; 
+         node != NULL; 
+         node = node->next) {
+      xmlChar *node_filename;
+
+      if (xmlIsBlankNode(node)) continue;
 
       if (node->type != XML_ELEMENT_NODE || strcmp(node->name, "plugin") != 0)
 	continue;
       node_filename = xmlGetProp(node, "filename");
       if (node_filename && !strcmp(info->filename, node_filename)) {
-	free(node_filename);
+	xmlFree(node_filename);
 	xmlReplaceNode(node, pluginnode);
 	xmlFreeNode(node);
 	break;
       }
-      if (node_filename) free(node_filename);
+      if (node_filename) xmlFree(node_filename);
     }
     /* node wasn't in document ... */
     if (!node)
-      xmlAddChild(pluginrc->root, pluginnode);
+      xmlAddChild(pluginrc->xmlRootNode, pluginnode);
     /* have to call this after adding node to document */
     xmlSetProp(pluginnode, "filename", info->filename);
   }
 
   filename = dia_config_filename("pluginrc");
-  xmlSaveFile(filename, pluginrc);
+  
+  xmlDiaSaveFile(filename,pluginrc);
+  
   g_free(filename);
   free_pluginrc();
 }
+
+
