@@ -65,23 +65,25 @@ struct _Flow {
 #define FLOW_ARROWWIDTH 0.5
 #define HANDLE_MOVE_TEXT (HANDLE_CUSTOM1)
 
-static void flow_move_handle(Flow *flow, Handle *handle,
-				   Point *to, HandleMoveReason reason);
-static void flow_move(Flow *flow, Point *to);
+static ObjectChange* flow_move_handle(Flow *flow, Handle *handle,
+				      Point *to, ConnectionPoint *cp,
+				      HandleMoveReason reason, 
+				      ModifierKeys modifiers);
+static ObjectChange* flow_move(Flow *flow, Point *to);
 static void flow_select(Flow *flow, Point *clicked_point,
 			      DiaRenderer *interactive_renderer);
 static void flow_draw(Flow *flow, DiaRenderer *renderer);
-static Object *flow_create(Point *startpoint,
+static DiaObject *flow_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
 				 Handle **handle2);
 static real flow_distance_from(Flow *flow, Point *point);
 static void flow_update_data(Flow *flow);
 static void flow_destroy(Flow *flow);
-static Object *flow_copy(Flow *flow);
+static DiaObject *flow_copy(Flow *flow);
 static void flow_save(Flow *flow, ObjectNode obj_node,
 			 const char *filename);
-static Object *flow_load(ObjectNode obj_node, int version,
+static DiaObject *flow_load(ObjectNode obj_node, int version,
 			    const char *filename);
 static DiaMenu *flow_get_object_menu(Flow *flow, Point *clickedpoint) ;
 
@@ -95,7 +97,7 @@ static ObjectTypeOps flow_type_ops =
   (ApplyDefaultsFunc)	NULL,
 } ;
 
-ObjectType flow_type =
+DiaObjectType flow_type =
 {
   "FS - Flow",         /* name */
   0,                   /* version */
@@ -203,9 +205,10 @@ flow_select(Flow *flow, Point *clicked_point,
   connection_update_handles(&flow->connection);
 }
 
-static void
+static ObjectChange*
 flow_move_handle(Flow *flow, Handle *handle,
-		 Point *to, HandleMoveReason reason)
+		 Point *to, ConnectionPoint *cp,
+		 HandleMoveReason reason, ModifierKeys modifiers)
 {
   Point p1, p2;
   Point *endpoints;
@@ -243,7 +246,8 @@ flow_move_handle(Flow *flow, Handle *handle,
        norm_mag = (real)sqrt( (double) point_dot( &p1, &p1 ) ) ;
     }
 
-    connection_move_handle(&flow->connection, handle->id, to, reason);
+    connection_move_handle(&flow->connection, handle->id, to, cp, 
+			   reason, modifiers);
 
     p2 = endpoints[1] ;
     point_sub( &p2, &endpoints[0] ) ;
@@ -264,9 +268,11 @@ flow_move_handle(Flow *flow, Handle *handle,
   }
 
   flow_update_data(flow);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 flow_move(Flow *flow, Point *to)
 {
   Point start_to_end;
@@ -285,6 +291,8 @@ flow_move(Flow *flow, Point *to)
   point_add(&flow->text->position, &delta);
   
   flow_update_data(flow);
+
+  return NULL;
 }
 
 static void
@@ -343,7 +351,7 @@ flow_draw(Flow *flow, DiaRenderer *renderer)
   text_draw(flow->text, renderer);
 }
 
-static Object *
+static DiaObject *
 flow_create(Point *startpoint,
 		  void *user_data,
 		  Handle **handle1,
@@ -352,7 +360,7 @@ flow_create(Point *startpoint,
   Flow *flow;
   ConnectionBBExtras *extra;
   Connection *conn;
-  Object *obj;
+  DiaObject *obj;
   Point p ;
   Point n ;
 
@@ -413,12 +421,12 @@ flow_destroy(Flow *flow)
   text_destroy(flow->text) ;
 }
 
-static Object *
+static DiaObject *
 flow_copy(Flow *flow)
 {
   Flow *newflow;
   Connection *conn, *newconn;
-  Object *newobj;
+  DiaObject *newobj;
   
   conn = &flow->connection;
   
@@ -442,7 +450,7 @@ static void
 flow_update_data(Flow *flow)
 {
   Connection *conn = &flow->connection;
-  Object *obj = &conn->object;
+  DiaObject *obj = &conn->object;
   Rectangle rect;
   Color* color ;
   
@@ -485,13 +493,13 @@ flow_save(Flow *flow, ObjectNode obj_node, const char *filename)
 		   flow->type);
 }
 
-static Object *
+static DiaObject *
 flow_load(ObjectNode obj_node, int version, const char *filename)
 {
   Flow *flow;
   AttributeNode attr;
   Connection *conn;
-  Object *obj;
+  DiaObject *obj;
   ConnectionBBExtras *extra;
 
   if (flow_font == NULL) {
@@ -539,7 +547,7 @@ flow_load(ObjectNode obj_node, int version, const char *filename)
 }
 
 static ObjectChange *
-flow_set_type_callback (Object* obj, Point* clicked, gpointer data)
+flow_set_type_callback (DiaObject* obj, Point* clicked, gpointer data)
 {
   ((Flow*)obj)->type = (int) data ;
   flow_update_defaults( (Flow*) obj, 1 ) ;
